@@ -1,7 +1,5 @@
 ##
-# Copyright 2012 Ghent University
-# Copyright 2012 Stijn De Weirdt
-# Copyright 2012 Kenneth Hoste
+# Copyright 2012-2014 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -26,14 +24,15 @@
 ##
 """
 Support for LAPACK as toolchain linear algebra library.
+
+@author: Stijn De Weirdt (Ghent University)
+@author: Kenneth Hoste (Ghent University)
 """
 
 from easybuild.tools.toolchain.linalg import LinAlg
 
 
 class Lapack(LinAlg):
-    """Trivial class
-        provides LAPACK
-    """
+    """Trivial class, provides LAPACK support."""
     LAPACK_MODULE_NAME = ['LAPACK']
     LAPACK_LIB = ['lapack']
