@@ -1,5 +1,5 @@
-##
-# Copyright 2009-2013 Ghent University
+# #
+# Copyright 2009-2014 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -21,7 +21,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
-##
+# #
 """
 Set of file tools.
 
@@ -31,38 +31,25 @@ Set of file tools.
 @author: Pieter De Baets (Ghent University)
 @author: Jens Timmerman (Ghent University)
 @author: Toon Willems (Ghent University)
+@author: Ward Poelmans (Ghent University)
 """
-import binascii
 import errno
 import os
 import re
 import shutil
-import signal
 import stat
-import subprocess
-import tempfile
 import time
 import urllib
 import zlib
 from vsc import fancylogger
 from vsc.utils.missing import all
 
-import easybuild.tools.build_log  # @UnusedImport (required to get an EasyBuildLog object from fancylogger.getLogger)
 import easybuild.tools.environment as env
-from easybuild.tools.asyncprocess import Popen, PIPE, STDOUT
-from easybuild.tools.asyncprocess import send_all, recv_some
+from easybuild.tools.build_log import print_msg  # import build_log must stay, to activate use of EasyBuildLog
+from easybuild.tools import run
 
 
 _log = fancylogger.getLogger('filetools', fname=False)
-errorsFoundInLog = 0
-
-# constants for strictness levels
-IGNORE = 'ignore'
-WARN = 'warn'
-ERROR = 'error'
-
-# default strictness level
-strictness = WARN
 
 # easyblock class prefix
 EASYBLOCK_CLASS_PREFIX = 'EB_'
@@ -104,24 +91,46 @@ STRING_ENCODING_CHARMAP = {
     r'~': "_tilde_",
 }
 
+try:
+    # preferred over md5/sha modules, but only available in Python 2.5 and more recent
+    import hashlib
+    md5_class = hashlib.md5
+    sha1_class = hashlib.sha1
+except ImportError:
+    import md5, sha
+    md5_class = md5.md5
+    sha1_class = sha.sha
+
 # default checksum for source and patch files
 DEFAULT_CHECKSUM = 'md5'
 
 # map of checksum types to checksum functions
 CHECKSUM_FUNCTIONS = {
-    'adler32': lambda p: '0x%s' % zlib.adler32(open(p, 'r').read()),
-    'crc32': lambda p: '0x%s' % binascii.crc32(open(p, 'r').read()),
+    'md5': lambda p: calc_block_checksum(p, md5_class()),
+    'sha1': lambda p: calc_block_checksum(p, sha1_class()),
+    'adler32': lambda p: calc_block_checksum(p, ZlibChecksum(zlib.adler32)),
+    'crc32': lambda p: calc_block_checksum(p, ZlibChecksum(zlib.crc32)),
     'size': lambda p: os.path.getsize(p),
 }
-try:
-    # preferred over md5/sha modules, but only available in Python 2.5 and more recent
-    import hashlib
-    CHECKSUM_FUNCTIONS['md5'] = lambda p: hashlib.md5(open(p, 'r').read()).hexdigest()
-    CHECKSUM_FUNCTIONS['sha1'] = lambda p: hashlib.sha1(open(p, 'r').read()).hexdigest()
-except ImportError:
-    import md5, sha
-    CHECKSUM_FUNCTIONS['md5'] = lambda p: md5.md5(open(p, 'r').read()).hexdigest()
-    CHECKSUM_FUNCTIONS['sha1'] = lambda p: sha.sha(open(p, 'r').read()).hexdigest()
+
+
+class ZlibChecksum(object):
+    """
+    wrapper class for adler32 and crc32 checksums to
+    match the interface of the hashlib module
+    """
+    def __init__(self, algorithm):
+        self.algorithm = algorithm
+        self.checksum = algorithm(r'')  # use the same starting point as the module
+        self.blocksize = 64  # The same as md5/sha1
+
+    def update(self, data):
+        """Calculates a new checksum using the old one and the new data"""
+        self.checksum = self.algorithm(data, self.checksum)
+
+    def hexdigest(self):
+        """Return hex string of the checksum"""
+        return '0x%s' % (self.checksum & 0xffffffff)
 
 
 def read_file(path, log_error=True):
@@ -167,16 +176,16 @@ def extract_file(fn, dest, cmd=None, extra_options=None, overwrite=False):
         _log.error("Can't extract file %s: no such file" % fn)
 
     if not os.path.isdir(dest):
-        ## try to create it
+        # try to create it
         try:
             os.makedirs(dest)
         except OSError, err:
             _log.exception("Can't extract file %s: directory %s can't be created: %err " % (fn, dest, err))
 
-    ## use absolute pathnames from now on
+    # use absolute pathnames from now on
     absDest = os.path.abspath(dest)
 
-    ## change working directory
+    # change working directory
     try:
         _log.debug("Unpacking %s in directory %s." % (fn, absDest))
         os.chdir(absDest)
@@ -270,6 +279,85 @@ def download_file(filename, url, path):
     return None
 
 
+def find_easyconfigs(path, ignore_dirs=None):
+    """
+    Find .eb easyconfig files in path
+    """
+    if os.path.isfile(path):
+        return [path]
+
+    if ignore_dirs is None:
+        ignore_dirs = []
+
+    # walk through the start directory, retain all files that end in .eb
+    files = []
+    path = os.path.abspath(path)
+    for dirpath, dirnames, filenames in os.walk(path, topdown=True):
+        for f in filenames:
+            if not f.endswith('.eb') or f == 'TEMPLATE.eb':
+                continue
+
+            spec = os.path.join(dirpath, f)
+            _log.debug("Found easyconfig %s" % spec)
+            files.append(spec)
+
+        # ignore subdirs specified to be ignored by replacing items in dirnames list used by os.walk
+        dirnames[:] = [d for d in dirnames if not d in ignore_dirs]
+
+    return files
+
+
+def search_file(paths, query, build_options=None, short=False):
+    """
+    Search for a particular file (only prints)
+    """
+    if build_options is None:
+        build_options = {}
+
+    ignore_dirs = build_options.get('ignore_dirs', ['.git', '.svn'])
+    if not isinstance(ignore_dirs, list):
+        _log.error("search_file: ignore_dirs (%s) should be of type list, not %s" % (ignore_dirs, type(ignore_dirs)))
+
+    silent = build_options.get('silent', False)
+
+    var_lines = []
+    hit_lines = []
+    var_index = 1
+    var = None
+    for path in paths:
+        hits = []
+        hit_in_path = False
+        print_msg("Searching (case-insensitive) for '%s' in %s " % (query, path), log=_log, silent=silent)
+
+        query = query.lower()
+        for (dirpath, dirnames, filenames) in os.walk(path, topdown=True):
+            for filename in filenames:
+                filename = os.path.join(dirpath, filename)
+                if filename.lower().find(query) != -1:
+                    if not hit_in_path:
+                        var = "CFGS%d" % var_index
+                        var_index += 1
+                        hit_in_path = True
+                    hits.append(filename)
+
+            # do not consider (certain) hidden directories
+            # note: we still need to consider e.g., .local !
+            # replace list elements using [:], so os.walk doesn't process deleted directories
+            # see http://stackoverflow.com/questions/13454164/os-walk-without-hidden-folders
+            dirnames[:] = [d for d in dirnames if not d in ignore_dirs]
+
+        if hits:
+            common_prefix = det_common_path_prefix(hits)
+            if short and common_prefix is not None and len(common_prefix) > len(var) * 2:
+                var_lines.append("%s=%s" % (var, common_prefix))
+                hit_lines.extend([" * %s" % os.path.join('$%s' % var, fn[len(common_prefix) + 1:]) for fn in hits])
+            else:
+                hit_lines.extend([" * %s" % fn for fn in hits])
+
+    for line in var_lines + hit_lines:
+        print_msg(line, log=_log, silent=silent, prefix=False)
+
+
 def compute_checksum(path, checksum_type=DEFAULT_CHECKSUM):
     """
     Compute checksum of specified file.
@@ -291,6 +379,28 @@ def compute_checksum(path, checksum_type=DEFAULT_CHECKSUM):
     return checksum
 
 
+def calc_block_checksum(path, algorithm):
+    """Calculate a checksum of a file by reading it into blocks"""
+    # We pick a blocksize of 16 MB: it's a multiple of the internal
+    # blocksize of md5/sha1 (64) and gave the best speed results
+    try:
+        # in hashlib, blocksize is a class parameter
+        blocksize = algorithm.blocksize * 262144  # 2^18
+    except AttributeError, err:
+        blocksize = 16777216  # 2^24
+    _log.debug("Using blocksize %s for calculating the checksum" % blocksize)
+
+    try:
+        f = open(path, 'rb')
+        for block in iter(lambda: f.read(blocksize), r''):
+            algorithm.update(block)
+        f.close()
+    except IOError, err:
+        _log.error("Failed to read %s: %s" % (path, err))
+
+    return algorithm.hexdigest()
+
+
 def verify_checksum(path, checksums):
     """
     Verify checksum of specified file.
@@ -333,7 +443,7 @@ def find_base_dir():
       expect only the first one to give the correct path
     """
     def get_local_dirs_purged():
-        ## e.g. always purge the log directory
+        # e.g. always purge the log directory
         ignoreDirs = ["easybuild"]
 
         lst = os.listdir(os.getcwd())
@@ -432,7 +542,7 @@ def apply_patch(patchFile, dest, fn=None, copy=False, level=None):
         _log.error("Can't patch directory %s: no such directory" % dest)
         return
 
-    ## copy missing files
+    # copy missing files
     if copy:
         try:
             shutil.copy2(patchFile, dest)
@@ -442,7 +552,7 @@ def apply_patch(patchFile, dest, fn=None, copy=False, level=None):
             _log.error("Failed to copy %s to dir %s: %s" % (patchFile, dest, err))
             return
 
-    ## use absolute paths
+    # use absolute paths
     apatch = os.path.abspath(patchFile)
     adest = os.path.abspath(dest)
 
@@ -479,7 +589,7 @@ def apply_patch(patchFile, dest, fn=None, copy=False, level=None):
 
         p = None
         for line in plusLines:
-            ## locate file by stripping of /
+            # locate file by stripping of /
             f = line.group('file')
             tf2 = f.split('/')
             n = len(tf2)
@@ -495,8 +605,8 @@ def apply_patch(patchFile, dest, fn=None, copy=False, level=None):
             else:
                 _log.debug('No match found for %s, trying next +++ line of patch file...' % f)
 
-        if p == None: # p can also be zero, so don't use "not p"
-            ## no match
+        if p is None:  # p can also be zero, so don't use "not p"
+            # no match
             _log.error("Can't determine patch level for patch %s from directory %s" % (patchFile, adest))
         else:
             _log.debug("Guessed patch level %d for patch %s" % (p, patchFile))
@@ -513,319 +623,6 @@ def apply_patch(patchFile, dest, fn=None, copy=False, level=None):
 
     return result
 
-def adjust_cmd(func):
-    """Make adjustments to given command, if required."""
-
-    def inner(cmd, *args, **kwargs):
-        # SuSE hack
-        # - profile is not resourced, and functions (e.g. module) is not inherited
-        if 'PROFILEREAD' in os.environ and (len(os.environ['PROFILEREAD']) > 0):
-            filepaths = ['/etc/profile.d/modules.sh']
-            extra = ''
-            for fp in filepaths:
-                if os.path.exists(fp):
-                    extra = ". %s &&%s" % (fp, extra)
-                else:
-                    _log.warning("Can't find file %s" % fp)
-
-            cmd = "%s %s" % (extra, cmd)
-
-        return func(cmd, *args, **kwargs)
-
-    return inner
-
-@adjust_cmd
-def run_cmd(cmd, log_ok=True, log_all=False, simple=False, inp=None, regexp=True, log_output=False, path=None):
-    """
-    Executes a command cmd
-    - returns exitcode and stdout+stderr (mixed)
-    - no input though stdin
-    - if log_ok or log_all are set -> will log.error if non-zero exit-code
-    - if simple is True -> instead of returning a tuple (output, ec) it will just return True or False signifying succes
-    - inp is the input given to the command
-    - regexp -> Regex used to check the output for errors. If True will use default (see parselogForError)
-    - if log_output is True -> all output of command will be logged to a tempfile
-    - path is the path run_cmd should chdir to before doing anything
-    """
-    try:
-        if path:
-            os.chdir(path)
-
-        _log.debug("run_cmd: running cmd %s (in %s)" % (cmd, os.getcwd()))
-    except:
-        _log.info("running cmd %s in non-existing directory, might fail!" % cmd)
-
-    ## Log command output
-    if log_output:
-        runLog = tempfile.NamedTemporaryFile(suffix='.log', prefix='easybuild-run_cmd-')
-        _log.debug('run_cmd: Command output will be logged to %s' % runLog.name)
-        runLog.write(cmd + "\n\n")
-    else:
-        runLog = None
-
-    readSize = 1024 * 8
-
-    try:
-        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
-                           stdin=subprocess.PIPE, close_fds=True, executable="/bin/bash")
-    except OSError, err:
-        _log.error("run_cmd init cmd %s failed:%s" % (cmd, err))
-    if inp:
-        p.stdin.write(inp)
-    p.stdin.close()
-
-    ec = p.poll()
-    stdouterr = ''
-    while ec < 0:
-        # need to read from time to time.
-        # - otherwise the stdout/stderr buffer gets filled and it all stops working
-        output = p.stdout.read(readSize)
-        if runLog:
-            runLog.write(output)
-        stdouterr += output
-        ec = p.poll()
-
-    # read remaining data (all of it)
-    stdouterr += p.stdout.read()
-
-    # not needed anymore. subprocess does this correct?
-    # ec=os.WEXITSTATUS(ec)
-
-    ## Command log output
-    if log_output:
-        runLog.close()
-
-    return parse_cmd_output(cmd, stdouterr, ec, simple, log_all, log_ok, regexp)
-
-
-@adjust_cmd
-def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, regexp=True, std_qa=None, path=None):
-    """
-    Executes a command cmd
-    - looks for questions and tries to answer based on qa dictionary
-    - returns exitcode and stdout+stderr (mixed)
-    - no input though stdin
-    - if log_ok or log_all are set -> will log.error if non-zero exit-code
-    - if simple is True -> instead of returning a tuple (output, ec) it will just return True or False signifying succes
-    - regexp -> Regex used to check the output for errors. If True will use default (see parselogForError)
-    - if log_output is True -> all output of command will be logged to a tempfile
-    - path is the path run_cmd should chdir to before doing anything
-    """
-    try:
-        if path:
-            os.chdir(path)
-
-        _log.debug("run_cmd_qa: running cmd %s (in %s)" % (cmd, os.getcwd()))
-    except:
-        _log.info("running cmd %s in non-existing directory, might fail!" % cmd)
-
-    # Part 1: process the QandA dictionary
-    # given initial set of Q and A (in dict), return dict of reg. exp. and A
-    #
-    # make regular expression that matches the string with
-    # - replace whitespace
-    # - replace newline
-
-    def escape_special(string):
-        return re.sub(r"([\+\?\(\)\[\]\*\.\\\$])" , r"\\\1", string)
-
-    split = '[\s\n]+'
-    regSplit = re.compile(r"" + split)
-
-    def process_QA(q, a):
-        splitq = [escape_special(x) for x in regSplit.split(q)]
-        regQtxt = split.join(splitq) + split.rstrip('+') + "*$"
-        ## add optional split at the end
-        if not a.endswith('\n'):
-            a += '\n'
-        regQ = re.compile(r"" + regQtxt)
-        if regQ.search(q):
-            return (a, regQ)
-        else:
-            _log.error("runqanda: Question %s converted in %s does not match itself" % (q, regQtxt))
-
-    newQA = {}
-    _log.debug("newQA: ")
-    for question, answer in qa.items():
-        (a, regQ) = process_QA(question, answer)
-        newQA[regQ] = a
-        _log.debug("newqa[%s]: %s" % (regQ.pattern, a))
-
-    newstdQA = {}
-    if std_qa:
-        for question, answer in std_qa.items():
-            regQ = re.compile(r"" + question + r"[\s\n]*$")
-            if not answer.endswith('\n'):
-                answer += '\n'
-            newstdQA[regQ] = answer
-            _log.debug("newstdQA[%s]: %s" % (regQ.pattern, answer))
-
-    new_no_qa = []
-    if no_qa:
-        # simple statements, can contain wildcards
-        new_no_qa = [re.compile(r"" + x + r"[\s\n]*$") for x in no_qa]
-
-    _log.debug("New noQandA list is: %s" % [x.pattern for x in new_no_qa])
-
-    # Part 2: Run the command and answer questions
-    # - this needs asynchronous stdout
-
-    ## Log command output
-    if log_all:
-        try:
-            runLog = tempfile.NamedTemporaryFile(suffix='.log', prefix='easybuild-cmdqa-')
-            _log.debug('run_cmd_qa: Command output will be logged to %s' % runLog.name)
-            runLog.write(cmd + "\n\n")
-        except IOError, err:
-            _log.error("Opening log file for Q&A failed: %s" % err)
-    else:
-        runLog = None
-
-    maxHitCount = 50
-
-    try:
-        p = Popen(cmd, shell=True, stdout=PIPE, stderr=STDOUT, stdin=PIPE, close_fds=True, executable="/bin/bash")
-    except OSError, err:
-        _log.error("run_cmd_qa init cmd %s failed:%s" % (cmd, err))
-
-    ec = p.poll()
-    stdoutErr = ''
-    oldLenOut = -1
-    hitCount = 0
-
-    while ec < 0:
-        # need to read from time to time.
-        # - otherwise the stdout/stderr buffer gets filled and it all stops working
-        try:
-            tmpOut = recv_some(p)
-            if runLog:
-                runLog.write(tmpOut)
-            stdoutErr += tmpOut
-        # recv_some may throw Exception
-        except (IOError, Exception), err:
-            _log.debug("run_cmd_qa cmd %s: read failed: %s" % (cmd, err))
-            tmpOut = None
-
-        hit = False
-        for q, a in newQA.items():
-            res = q.search(stdoutErr)
-            if tmpOut and res:
-                fa = a % res.groupdict()
-                _log.debug("run_cmd_qa answer %s question %s out %s" % (fa, q.pattern, stdoutErr[-50:]))
-                send_all(p, fa)
-                hit = True
-                break
-        if not hit:
-            for q, a in newstdQA.items():
-                res = q.search(stdoutErr)
-                if tmpOut and res:
-                    fa = a % res.groupdict()
-                    _log.debug("run_cmd_qa answer %s standard question %s out %s" % (fa, q.pattern, stdoutErr[-50:]))
-                    send_all(p, fa)
-                    hit = True
-                    break
-            if not hit:
-                if len(stdoutErr) > oldLenOut:
-                    oldLenOut = len(stdoutErr)
-                else:
-                    noqa = False
-                    for r in new_no_qa:
-                        if r.search(stdoutErr):
-                            _log.debug("runqanda: noQandA found for out %s" % stdoutErr[-50:])
-                            noqa = True
-                    if not noqa:
-                        hitCount += 1
-            else:
-                hitCount = 0
-        else:
-            hitCount = 0
-
-        if hitCount > maxHitCount:
-            # explicitly kill the child process before exiting
-            try:
-                os.killpg(p.pid, signal.SIGKILL)
-                os.kill(p.pid, signal.SIGKILL)
-            except OSError, err:
-                _log.debug("run_cmd_qa exception caught when killing child process: %s" % err)
-            _log.debug("run_cmd_qa: full stdouterr: %s" % stdoutErr)
-            _log.error("run_cmd_qa: cmd %s : Max nohits %s reached: end of output %s" % (cmd,
-                                                                                    maxHitCount,
-                                                                                    stdoutErr[-500:]
-                                                                                    ))
-
-        # the sleep below is required to avoid exiting on unknown 'questions' too early (see above)
-        time.sleep(1)
-        ec = p.poll()
-
-    # Process stopped. Read all remaining data
-    try:
-        if p.stdout:
-            readTxt = p.stdout.read()
-            stdoutErr += readTxt
-            if runLog:
-                runLog.write(readTxt)
-    except IOError, err:
-        _log.debug("runqanda cmd %s: remaining data read failed: %s" % (cmd, err))
-
-    # Not needed anymore. Subprocess does this correct?
-    # ec=os.WEXITSTATUS(ec)
-
-    return parse_cmd_output(cmd, stdoutErr, ec, simple, log_all, log_ok, regexp)
-
-
-def parse_cmd_output(cmd, stdouterr, ec, simple, log_all, log_ok, regexp):
-    """
-    will parse and perform error checks based on strictness setting
-    """
-    if strictness == IGNORE:
-        check_ec = False
-        use_regexp = False
-    elif strictness == WARN:
-        check_ec = True
-        use_regexp = False
-    elif strictness == ERROR:
-        check_ec = True
-        use_regexp = True
-    else:
-        _log.error("invalid strictness setting: %s" % strictness)
-
-    # allow for overriding the regexp setting
-    if not regexp:
-        use_regexp = False
-
-    if ec and (log_all or log_ok):
-        # We don't want to error if the user doesn't care
-        if check_ec:
-            _log.error('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
-        else:
-            _log.warn('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
-
-    if not ec:
-        if log_all:
-            _log.info('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
-        else:
-            _log.debug('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
-
-    # parse the stdout/stderr for errors when strictness dictates this or when regexp is passed in
-    if use_regexp or regexp:
-        res = parse_log_for_error(stdouterr, regexp, msg="Command used: %s" % cmd)
-        if len(res) > 0:
-            message = "Found %s errors in command output (output: %s)" % (len(res), ", ".join([r[0] for r in res]))
-            if use_regexp:
-                _log.error(message)
-            else:
-                _log.warn(message)
-
-    if simple:
-        if ec:
-            # If the user does not care -> will return true
-            return not check_ec
-        else:
-            return True
-    else:
-        # Because we are not running in simple mode, we return the output and ec to the user
-        return (stdouterr, ec)
-
 
 def modify_env(old, new):
     """
@@ -839,11 +636,11 @@ def convert_name(name, upper=False):
     """
     Converts name so it can be used as variable name
     """
-    ## no regexps
+    # no regexps
     charmap = {
-         '+':'plus',
-         '-':'min'
-        }
+        '+': 'plus',
+        '-': 'min'
+    }
     for ch, new in charmap.items():
         name = name.replace(ch, new)
 
@@ -853,42 +650,6 @@ def convert_name(name, upper=False):
         return name
 
 
-def parse_log_for_error(txt, regExp=None, stdout=True, msg=None):
-    """
-    txt is multiline string.
-    - in memory
-    regExp is a one-line regular expression
-    - default
-    """
-    global errorsFoundInLog
-
-    if regExp and type(regExp) == bool:
-        regExp = r"(?<![(,-]|\w)(?:error|segmentation fault|failed)(?![(,-]|\.?\w)"
-        _log.debug('Using default regular expression: %s' % regExp)
-    elif type(regExp) == str:
-        pass
-    else:
-        _log.error("parselogForError no valid regExp used: %s" % regExp)
-
-    reg = re.compile(regExp, re.I)
-
-    res = []
-    for l in txt.split('\n'):
-        r = reg.search(l)
-        if r:
-            res.append([l, r.groups()])
-            errorsFoundInLog += 1
-
-    if stdout and res:
-        if msg:
-            _log.info("parseLogError msg: %s" % msg)
-        _log.info("parseLogError (some may be harmless) regExp %s found:\n%s" % (regExp,
-                                                                              '\n'.join([x[0] for x in res])
-                                                                              ))
-
-    return res
-
-
 def adjust_permissions(name, permissionBits, add=True, onlyfiles=False, onlydirs=False, recursive=True,
                        group_id=None, relative=True, ignore_errors=False):
     """
@@ -958,10 +719,10 @@ def adjust_permissions(name, permissionBits, add=True, onlyfiles=False, onlydirs
     fail_ratio = fail_cnt / float(len(allpaths))
     max_fail_ratio = 0.5
     if fail_ratio > max_fail_ratio:
-        _log.error("%.2f%% of permissions/owner operations failed (more than %.2f%%), something must be wrong..." % \
-                  (100*fail_ratio, 100*max_fail_ratio))
+        _log.error("%.2f%% of permissions/owner operations failed (more than %.2f%%), something must be wrong..." %
+                  (100 * fail_ratio, 100 * max_fail_ratio))
     elif fail_cnt > 0:
-        _log.debug("%.2f%% of permissions/owner operations failed, ignoring that..." % (100*fail_ratio))
+        _log.debug("%.2f%% of permissions/owner operations failed, ignoring that..." % (100 * fail_ratio))
 
 
 def patch_perl_script_autoflush(path):
@@ -988,7 +749,7 @@ def mkdir(directory, parents=False):
     """
     Create a directory
     Directory is the path to create
-    
+
     When parents is True then no error if directory already exists
     and make parent directories as needed (cfr. mkdir -p)
     """
@@ -1001,7 +762,7 @@ def mkdir(directory, parents=False):
                 _log.debug("Directory %s already exitst" % directory)
             else:
                 _log.error("Failed to create directory %s: %s" % (directory, err))
-    else:#not parrents
+    else:  # not parents
         try:
             os.mkdir(directory)
             _log.debug("Succesfully created directory %s" % directory)
@@ -1011,11 +772,12 @@ def mkdir(directory, parents=False):
             else:
                 _log.error("Failed to create directory %s: %s" % (directory, err))
 
+
 def rmtree2(path, n=3):
     """Wrapper around shutil.rmtree to make it more robust when used on NFS mounted file systems."""
 
     ok = False
-    for i in range(0,n):
+    for i in range(0, n):
         try:
             shutil.rmtree(path)
             ok = True
@@ -1028,11 +790,23 @@ def rmtree2(path, n=3):
     else:
         _log.info("Path %s successfully removed." % path)
 
+
+def cleanup(logfile, tempdir, testing):
+    """Cleanup the specified log file and the tmp directory"""
+    if not testing and logfile is not None:
+        os.remove(logfile)
+        print_msg('temporary log file %s has been removed.' % (logfile), log=None, silent=testing)
+
+    if not testing and tempdir is not None:
+        shutil.rmtree(tempdir, ignore_errors=True)
+        print_msg('temporary directory %s has been removed.' % (tempdir), log=None, silent=testing)
+
+
 def copytree(src, dst, symlinks=False, ignore=None):
     """
     Copied from Lib/shutil.py in python 2.7, since we need this to work for python2.4 aswell
     and this code can be improved...
-    
+
     Recursively copy a directory tree using copy2().
 
     The destination directory must not already exist.
@@ -1061,7 +835,7 @@ def copytree(src, dst, symlinks=False, ignore=None):
     class Error(EnvironmentError):
         pass
     try:
-        WindowsError #@UndefinedVariable
+        WindowsError  # @UndefinedVariable
     except NameError:
         WindowsError = None
 
@@ -1104,6 +878,7 @@ def copytree(src, dst, symlinks=False, ignore=None):
     if errors:
         raise Error, errors
 
+
 def encode_string(name):
     """
     This encoding function handles funky software names ad infinitum, like:
@@ -1116,7 +891,7 @@ def encode_string(name):
     * http://celldesigner.org/help/CDH_Species_01.html
     * http://research.cs.berkeley.edu/project/sbp/darcsrepo-no-longer-updated/src/edu/berkeley/sbp/misc/ReflectiveWalker.java
     and can be extended freely as per ISO/IEC 10646:2012 / Unicode 6.1 names:
-    * http://www.unicode.org/versions/Unicode6.1.0/ 
+    * http://www.unicode.org/versions/Unicode6.1.0/
     For readability of >2 words, it is suggested to use _CamelCase_ style.
     So, yes, '_GreekSmallLetterEtaWithPsiliAndOxia_' *could* indeed be a fully
     valid software name; software "electron" in the original spelling anyone? ;-)
@@ -1127,6 +902,7 @@ def encode_string(name):
     result = ''.join(map(lambda x: STRING_ENCODING_CHARMAP.get(x, x), name))
     return result
 
+
 def decode_string(name):
     """Decoding function to revert result of encode_string."""
     result = name
@@ -1134,10 +910,12 @@ def decode_string(name):
         result = re.sub(escaped_char, char, result)
     return result
 
+
 def encode_class_name(name):
     """return encoded version of class name"""
     return EASYBLOCK_CLASS_PREFIX + encode_string(name)
 
+
 def decode_class_name(name):
     """Return decoded version of class name."""
     if not name.startswith(EASYBLOCK_CLASS_PREFIX):
@@ -1146,3 +924,19 @@ def decode_class_name(name):
     else:
         name = name[len(EASYBLOCK_CLASS_PREFIX):]
         return decode_string(name)
+
+
+def run_cmd(cmd, log_ok=True, log_all=False, simple=False, inp=None, regexp=True, log_output=False, path=None):
+    """Legacy wrapper/placeholder for run.run_cmd"""
+    return run.run_cmd(cmd, log_ok=log_ok, log_all=log_all, simple=simple,
+                       inp=inp, regexp=regexp, log_output=log_output, path=path)
+
+
+def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, regexp=True, std_qa=None, path=None):
+    """Legacy wrapper/placeholder for run.run_cmd_qa"""
+    return run.run_cmd_qa(cmd, qa, no_qa=no_qa, log_ok=log_ok, log_all=log_all,
+                          simple=simple, regexp=regexp, std_qa=std_qa, path=path)
+
+def parse_log_for_error(txt, regExp=None, stdout=True, msg=None):
+    """Legacy wrapper/placeholder for run.parse_log_for_error"""
+    return run.parse_log_for_error(txt, regExp=regExp, stdout=stdout, msg=msg)
