@@ -1,6 +1,6 @@
 #!/usr/bin/python
 # #
-# Copyright 2012-2013 Ghent University
+# Copyright 2012-2014 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -47,13 +47,16 @@ import test.framework.easyconfigparser as ep
 import test.framework.easyconfigformat as ef
 import test.framework.easyconfigversion as ev
 import test.framework.filetools as f
+import test.framework.format_convert as f_c
 import test.framework.github as g
 import test.framework.license as l
 import test.framework.module_generator as mg
 import test.framework.modules as m
+import test.framework.modulestool as mt
 import test.framework.options as o
 import test.framework.repository as r
 import test.framework.robot as robot
+import test.framework.run as run
 import test.framework.systemtools as s
 import test.framework.toolchain as tc
 import test.framework.toolchainvariables as tcv
@@ -70,7 +73,7 @@ for test_fn in [fn, os.path.join(testdir, 'test')]:
     try:
         open(fn, 'w').write('test')
     except IOError, err:
-        sys.stderr.write("ERROR: Can't write to temporary file %s, set $TMPDIR to a writeable directory" % (fn, msg))
+        sys.stderr.write("ERROR: Can't write to temporary file %s, set $TMPDIR to a writeable directory (%s)" % (fn, err))
         sys.exit(1)
 os.remove(fn)
 shutil.rmtree(testdir)
@@ -85,7 +88,7 @@ log.setLevelName('DEBUG')
 
 # call suite() for each module and then run them all
 # note: make sure the options unit tests run first, to avoid running some of them with a readily initialized config
-tests = [o, r, ef, ev, ep, e, mg, m, f, a, robot, b, v, g, tcv, tc, t, c, s, l]
+tests = [o, r, ef, ev, ep, e, mg, m, mt, f, run, a, robot, b, v, g, tcv, tc, t, c, s, l, f_c]
 SUITE = unittest.TestSuite([x.suite() for x in tests])
 
 # uses XMLTestRunner if possible, so we can output an XML file that can be supplied to Jenkins
