@@ -1,5 +1,5 @@
-##
-# Copyright 2013 Ghent University
+# #
+# Copyright 2013-2014 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -21,7 +21,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
-##
+# #
 """
 Toy build unit test
 
@@ -100,6 +100,30 @@ class ToyBuildTest(TestCase):
                 print "err: %s" % err
             self.assertTrue(res)
 
+    def check_toy(self, installpath, outtxt, version='0.0', versionprefix='', versionsuffix=''):
+        """Check whether toy build succeeded."""
+
+        full_version = ''.join([versionprefix, version, versionsuffix])
+
+        # if the module exists, it should be fine
+        toy_module = os.path.join(installpath, 'modules', 'all', 'toy', full_version)
+        self.assertTrue(os.path.exists(toy_module), "module for toy build toy/%s found" % full_version)
+
+        # check for success
+        success = re.compile("COMPLETED: Installation ended successfully")
+        self.assertTrue(success.search(outtxt))
+
+        # make sure installation log file and easyconfig file are copied to install dir
+        software_path = os.path.join(installpath, 'software', 'toy', full_version)
+        install_log_path_pattern = os.path.join(software_path, 'easybuild', 'easybuild-toy-%s*.log' % version)
+        self.assertTrue(len(glob.glob(install_log_path_pattern)) == 1, "Found 1 file at %s" % install_log_path_pattern)
+
+        ec_file_path = os.path.join(software_path, 'easybuild', 'toy-%s.eb' % full_version)
+        self.assertTrue(os.path.exists(ec_file_path))
+
+        devel_module_path = os.path.join(software_path, 'easybuild', 'toy-%s-easybuild-devel' % full_version)
+        self.assertTrue(os.path.exists(devel_module_path))
+
     def test_toy_build(self):
         """Perform a toy build."""
         args = [
@@ -114,27 +138,46 @@ class ToyBuildTest(TestCase):
                ]
         try:
             main((args, self.dummylogfn, True))
-        except (SystemExit, Exception), err:
-            print err
+        except SystemExit:
+            pass
+        except Exception, err:
+            print "err: %s" % err
         outtxt = read_file(self.logfile)
 
-        # if the module exists, it should be fine
-        toy_module = os.path.join(self.installpath, 'modules', 'all', 'toy', '0.0')
-        self.assertTrue(os.path.exists(toy_module), "module for toy build toy/0.0 found")
+        self.check_toy(self.installpath, outtxt)
 
-        # check for success
-        success = re.compile("COMPLETED: Installation ended successfully")
-        self.assertTrue(success.search(outtxt))
+    def test_toy_build_formatv2(self):
+        """Perform a toy build (format v2)."""
+        # set $MODULEPATH such that modules for specified dependencies are found
+        modulepath = os.environ.get('MODULEPATH')
+        os.environ['MODULEPATH'] = os.path.abspath(os.path.join(os.path.dirname(__file__), 'modules'))
 
-        # make sure installation log file and easyconfig file are copied to install dir
-        install_log_path_pattern = os.path.join(self.installpath, 'software', 'toy', '0.0', 'easybuild', 'easybuild-toy-0.0-*.log')
-        self.assertTrue(len(glob.glob(install_log_path_pattern)) == 1)
+        args = [
+            os.path.join(os.path.dirname(__file__), 'easyconfigs', 'v2.0', 'toy.eb'),
+            '--sourcepath=%s' % self.sourcepath,
+            '--buildpath=%s' % self.buildpath,
+            '--installpath=%s' % self.installpath,
+            '--debug',
+            '--unittest-file=%s' % self.logfile,
+            '--force',
+            '--robot=%s' % os.pathsep.join([self.buildpath, os.path.dirname(__file__)]),
+            '--software-version=0.0',
+            '--toolchain=dummy,dummy',
+            '--experimental',
+        ]
+        try:
+            main((args, self.dummylogfn, True))
+        except SystemExit:
+            pass
+        outtxt = read_file(self.logfile)
 
-        ec_file_path = os.path.join(self.installpath, 'software', 'toy', '0.0', 'easybuild', 'toy-0.0.eb')
-        self.assertTrue(os.path.exists(ec_file_path))
+        self.check_toy(self.installpath, outtxt)
 
-        devel_module_path = os.path.join(self.installpath, 'software', 'toy', '0.0', 'easybuild', 'toy-0.0-easybuild-devel')
-        self.assertTrue(os.path.exists(devel_module_path))
+        # restore
+        if modulepath is not None:
+            os.environ['MODULEPATH'] = modulepath
+        else:
+            del os.environ['MODULEPATH']
 
     def test_toy_build_with_blocks(self):
         """Test a toy build with multiple blocks."""
@@ -158,12 +201,18 @@ class ToyBuildTest(TestCase):
                ]
         try:
             main((args, self.dummylogfn, True))
-        except (SystemExit, Exception), err:
-            print err
+        except SystemExit:
+            pass
+        except Exception, err:
+            print "err: %s" % err
+        outtxt = read_file(self.logfile)
 
-        for toy_version in ['0.0-somesuffix', 'someprefix-0.0-somesuffix']:
-            toy_module = os.path.join(self.installpath, 'modules', 'all', 'toy', toy_version)
-            self.assertTrue(os.path.exists(toy_module), "module for toy/%s found" % toy_version)
+        for toy_prefix, toy_version, toy_suffix in [
+            ('', '0.0', '-somesuffix'),
+            ('someprefix-', '0.0', '-somesuffix')
+        ]:
+            self.check_toy(self.installpath, outtxt, version=toy_version,
+                           versionprefix=toy_prefix, versionsuffix=toy_suffix)
 
         # cleanup
         shutil.rmtree(tmpdir)
